#! /bin/csh
#
# file uico.csh
#      ========
#
#            03-May-2000 added Karl's suggestions to reflect all formats. U.K.
# version 3, 06-Jun-99 added more options for help. Urs Kradolfer
# version 2, 20-Aug-98
#
# User interface for codeco calls
# K. Stammler, 19-Aug-98

if  ("$1" == "-h" || "$1" == "--help" || "$1" == "-?" || "$1" == "?")  then
	echo "Usage: $0 <inpfile> <infmt> <outfile> <outfmt>"
	echo "   Parameters are prompted if not passed."
	exit
endif

# get path to codeco
#set copath=$0:h
#if  ("$copath" == "$0" || "$copath" == "./")  set copath="."
set copath=/gse/codeco/codeco3
set codeco=$copath/bin/call_codeco.csh


# get parameters from command line
set inpfile=$1
set infmt=$2
set outfile=$3
set outfmt=$4

# prompt for all parameters not passed

# input file
if  ("$inpfile" == "")  then
	while  (1 > 0)
		echo "Enter the name of the file to be converted."
		echo -n "Input file: "
		set inpfile=$<
		if  ("$inpfile" == "")  continue
		if  (! -e "$inpfile")  then
			echo "Input file $inpfile not found.  Please specify another file."
			continue
		endif
		break
	end
endif

# input format
set validform = ( auto gse sac saca seed css )
if  ("$infmt" == "")  then
	set found=0
	while  ($found == 0)
		echo "Enter input format.  The following choices are possible:"
		echo "auto     automatic detection of input format (may fail)"
		echo "gse      any GSE1/2 format"
		echo "sac      SAC-binary or SAC-ascii format"
		echo "saca     SAC-binary or SAC-ascii format"
		echo "seed     MiniSEED (4096-blocks) format"
		echo "css      CSS (2.8/3.0) format"
		echo -n "Input format: "
		set infmt=$<
		set found=0
		foreach f ($validform)
			if  ("$f" == "$infmt")  then
				set found=1
				break
			endif
		end
	end
endif

# output file
if  ("$outfile" == "")  then
	while  (1 > 0)
		echo "Enter name of output file where the results of the conversion will go."
		echo -n "Output file: "
		set outfile=$<
		if  ("$outfile" == "")  continue
		if  (-e $outfile)  then
			while  (1 > 0)
				echo "File $outfile is already existing."
				echo -n "Do you want to overwrite it ? [y/n]: "
				set res=$<
				if  ("$res" == "n")  exit
				if  ("$res" == "y")  break
			end
			\rm $outfile
		endif
		touch $outfile
		if  (! -e $outfile)  then
			echo "No write permission for $outfile."
			continue
		endif
		\rm $outfile
		break
	end
endif

# output format
set validform = ( intv-0 intv-1 intv-2 cmp6-0 cmp6-1 cmp6-2 cmp7-0 cmp7-1 \
	cmp7-2 cmp8-0 cmp8-1 cmp8-2 int cm6 cm7 cm8 sac saca seed css2 css3 )
if  ("$outfmt" == "")  then
	set found=0
	while  ($found == 0)
		echo "Enter output format.  The following choices are possible:"
		echo "intv-0   GSE1.0 uncompressed integer, no differences (printable)"
		echo "intv-1   GSE1.0 uncompressed integer, first differences (printable)"
		echo "intv-2   GSE1.0 uncompressed integer, second differences (printable)"
		echo "cmp6-0   GSE1.0 6-bit compressed, no differences (printable)"
		echo "cmp6-1   GSE1.0 6-bit compressed, first differences (printable)"
		echo "cmp6-2   GSE1.0 6-bit compressed, second differences (printable)"
		echo "cmp7-0   GSE1.0 7-bit compressed, no differences (binary)"
		echo "cmp7-1   GSE1.0 7-bit compressed, first differences (binary)"
		echo "cmp7-2   GSE1.0 7-bit compressed, second differences (binary)"
		echo "cmp8-0   GSE1.0 8-bit compressed, no differences (binary)"
		echo "cmp8-1   GSE1.0 8-bit compressed, first differences (binary)"
		echo "cmp8-2   GSE1.0 8-bit compressed, second differences (binary)"
		echo "int      GSE2.0 uncompressed integer, no differences (printable)"
		echo "cm6      GSE2.0 6-bit compressed, second differences (printable)"
		echo "cm7      GSE2.0 7-bit compressed, second differences (binary)"
		echo "cm8      GSE2.0 8-bit compressed, second differences (binary)"
		echo "sac      SAC format (binary)"
		echo "saca     SAC ASCII format (printable)"
		echo "seed     Mini-SEED, Steim1-compressed (binary)" 
		echo "css2     'CSS-2.8' format" 
		echo "css3     'CSS-3.0' format" 
		echo -n "Output format: "
		set outfmt=$<
		set found=0
		foreach f ($validform)
			if  ("$f" == "$outfmt")  then
				set found=1
				break
			endif
		end
	end
endif

#set echo
$codeco $inpfile $infmt $outfile $outfmt

