      program codeco
c             ======
c
c     Modified some SEED stuff. 26Nov99 K.Koch
c
c     version 3.0, 4-Nov-98
c     version 98.001,  8-May-98
c
c------------------------------------------------------------------------------
c
c     Urs Kradolfer, 17.1.90, revised: 4. August 1990, 21. April 1991
c                    major revision: 4. September 1991
c                    revised 28.1.92 and tested on HP-UX, SUN and VMS
c
c     Karl Koch, 10.5.95, added GSE2.0 support
c
c     Karl Koch / Klaus Stammler, major revision 17.-21.8.98:
c        bug fixes:
c           truncation of milliseconds at GSE2-output fixed,
c           correct handling of leap years
c        separation in modules
c        header variables put in common block
c        support for SAC I/O, Mini-SEED-output
c        design of Makefiles
c        implementation of test suite using SH package
c        tested on:
c           HP-UX (f90)
c           Linux (g77 version 2.7.2)
c           [SunOS (g77 on Solaris 2.5.1)]
c
c  18.05.95/uk
c   - Some format-stuff modified and channel-ID in gse1to2 modified.
c   6.07.95/uk
c   - Bug in subr. gse2to1 removed.
c  11.11.95 / uk
c   - Bug in GSE2.0 format removed (hang and vang were wrong for some channels)
c  16.11.95 / uk
c   - Format of CHK2 (checksum GSE2.0) corrected. 
c  17.10.97 / kk
c   - Bug in GSE2.0 format fixed (iunits were not converted)
c   7.05.98 / kk
c   - Bug in GSE2.0 format fixed (INT included removal of 2nd difference)
c   - Also included error checking whether waveform length exceeds
c     program dimensions, i.e. c_sigsize
c   8.05.98 / kk
c   - Pulled the input/output parts into the subroutines GSEIN & GSEOUT
c  21.8.98 / kk/ks
c   - see above
c
c------------------------------------------------------------------------------
c
c     Program to convert GSE-datafiles into other formats.
c
c     NOTE: This version now supports both the 'old' GSE-format and
c           the 'new' GSE2.0 format 
c
c     NOTE: the 'old' GSE-format and the new GSE2.0 format do not contain the
c           same information on the header-lines. Therefore some information
c           may get lost if converting from old to new GSE-format or
c           vice versa.
c
c
c     Current array-dimensions:
c           The array dimensions are defined in 'codeco_common.f':
c           Signal (IY):     600000 samples        'c_sigsize'
c           Character-array: x 6 (6 times larger)  'c_bufsize'
c
c     This program (or its subroutines) may be used to decompress
c     data files which are compressed with the GSE-algorithms.
c     The Swiss Seismological Service uses these algorithms as well.
c
c --> See the remarks on authorship of the compression/decompression
c     routines!
c
c     format numbers:
c     1      GSE1.0
c     2      GSE2.x
c     -1     SAC
c     -2     SAC ASCII
c     -3     Mini-SEED
c     -11    CSS-2.8
c     -12    CSS-3.0

c     -- use no default types
      implicit none

c     -- include common block of header variables (hdr_...) and constants (c_...)
      include 'codeco_common.f'

c     -- constants
c     -- array dimensions 'c_sigsize' (former 'isignalsize') and
c     -- 'c_bufsize' (former 'ibufsize') are defined in 'codeco_common.f'

c     -- variables
      integer*4 iy(c_sigsize)
      integer*4 ichecksum
      integer*4 iwaveform
      integer*4 ierr
      integer       i                  ! counter
      logical       is_sac_file        ! input file is a SAC file
      logical       is_seed_file       ! input file is a MiniSEED file
      logical       is_css_file        ! input file is a CSS file
      integer*4     usr_ofmtnum        ! format number set by user
      integer*4     usr_odiff          ! no. of differences set by user
      character*10  usr_ofmtname       ! name of output format set by user
      character*10  usr_ifmtname       ! name of input format set by user
      character*10  usr_flags          ! program execution flags
      character*1 cbuf(c_bufsize)
      character infile*80,outfile*80

c     -- functions
      integer       trimlen

c     -- executable code

      hdr_debug = 0

      call init_hdrvars

c     -- hp-style prompt:      write(6,'(a,nn)')'Enter name of inputfile: '
c     -- vms-style prompt:     write(6,'(a)')'$Enter name of inputfile: '
c     -- prompt for parameters
      write (*,*) 'Enter name of inputfile: '
      read (5,'(a)') infile
      write (*,*) 'Enter type of inputfile (sac,saca,seed,css,gse,',
     &            'auto): '
      read (5,'(a)') usr_ifmtname
      write (*,*) 'Enter name of outputfile: '
      read (5,'(a)') outfile
      write (*,*) 'Possible output formats are:'
      write (*,*) 'INTn  (n=1,...,9)  '
      write (*,*) 'INTV  integer variable (`free-format`)'
      write (*,*) 'INT   GSE2.0 integer format'
      write (*,*) 'CMPn  (n=6, 7, or 8 bit compression)'
      write (*,*) 'CMn   (n=6, or 8 bit compression) in GSE2.0 format'
      write (*,*) 'SAC   SAC (binary format)'
      write (*,*) 'SACA  SAC (ASCII format)'
      write (*,*) 'SEED  Mini-SEED format'
      write (*,*) 'CSS2  CSS-2.8 format'
      write (*,*) 'CSS3  CSS-3.0 format'
      write (*,*) 'Enter desired output format: '
      read (5,'(a)') usr_ofmtname

c     -- make format names uppercase
      call casefold( usr_ofmtname )
      call casefold( usr_ifmtname )

c     -- read flags if any there
      call get_flags( usr_ifmtname, usr_flags )
      do  i=1,trimlen(usr_flags)
         if  (usr_flags(i:i) .eq. '0')  hdr_debug = 0
         if  (usr_flags(i:i) .eq. '1')  hdr_debug = 1
         if  (usr_flags(i:i) .eq. '2')  hdr_debug = 2
         if  (usr_flags(i:i) .eq. '3')  hdr_debug = 3
         if  (usr_flags(i:i) .eq. '4')  hdr_debug = 4
         if  (usr_flags(i:i) .eq. '5')  hdr_debug = 5
         if  (usr_flags(i:i) .eq. '6')  hdr_debug = 6
      enddo

c     -- set/request format numbers and order of differences
      if  (usr_ofmtname .eq. 'SAC')  then
         usr_ofmtnum = -1
         usr_odiff = 0
      else if  (usr_ofmtname .eq. 'SACA')  then
         usr_ofmtnum = -2
         usr_odiff = 0
      else if  (usr_ofmtname(1:4) .eq. 'SEED')  then
         usr_ofmtnum = -3
         usr_odiff = 0
      else if  (usr_ofmtname(1:3) .eq. 'CSS')  then
         if  (usr_ofmtname(1:4) .eq. 'CSS2') usr_ofmtnum = -11
         if  (usr_ofmtname(1:4) .eq. 'CSS3') usr_ofmtnum = -12
         usr_odiff = 0
      else if (usr_ofmtname(4:4).eq.' ') then
         usr_ofmtnum = 2
         usr_odiff = 0
         if (usr_ofmtname(1:2).eq.'CM') usr_odiff = 2
      else
         usr_ofmtnum = 1
         write(6,*)'Enter # of differences (0, 1 or 2): '
         read(5,*) usr_odiff
      endif

      if  (hdr_debug .gt. 3)  write (*,*)
     &   '-- outfmt name, no, diff: ',
     &   usr_ofmtname, usr_ofmtnum, usr_odiff

c     -- check for SAC input file if automatic detection is selected
      is_sac_file = .false.
      is_seed_file= .false.
      is_css_file = .false.
      if  (usr_ifmtname .eq. 'AUTO')  then
         if  (hdr_debug .gt. 2)  write (*,*)
     &      '-- autodetect: check for CSS'
         open(23,file=infile,status='old')
         call cssin( 23, iy, ierr )
         close(23)
         if (ierr.eq.0) then
            is_css_file=.true.
            if  (hdr_debug .gt. 2)  write (*,*)
     &         '-- is_css_file, ierr: ', is_css_file, ierr
            goto 1
         endif
         if  (hdr_debug .gt. 2)  write (*,*)
     &      '-- autodetect: check for MiniSEED'
         call mseedin( infile, iy, ierr )
         if (ierr.eq.0) then
            is_seed_file = .true.
            if  (hdr_debug .gt. 2)  write (*,*)
     &         '-- is_seed_file, ierr: ', is_seed_file, ierr
            goto 1
         endif
         if  (hdr_debug .gt. 2)  write (*,*)
     &      '-- autodetect: check for SAC'
         call sacin( infile, iy, ierr )
         if (ierr.eq.0) then
            is_sac_file = .true.
            if  (hdr_debug .gt. 2)  write (*,*)
     &         '-- is_sac_file, ierr: ', is_sac_file, ierr
            goto 1
         endif
         
      else if  (usr_ifmtname(1:3) .eq. 'SAC')  then
         is_sac_file = .true.
      else if  (usr_ifmtname(1:4) .eq. 'SEED')  then
         is_seed_file = .true.
      else if  (usr_ifmtname(1:3) .eq. 'CSS')  then
         is_css_file = .true.
      endif

1     continue
c     -- open input and output files
      open(4,file=outfile,status='unknown')
chp   open(3,file=infile,status='old',readonly)
      open(3,file=infile,status='old')

c     -- loop all traces in the input file
      iwaveform = 0
10    continue

c        -- reset header variables
         call init_hdrvars
         hdr_ofmtname = usr_ofmtname
         hdr_ofmtnum = usr_ofmtnum
         hdr_odiff = usr_odiff

c        -- read next trace from file, this sets hdr_...-variables
         if  (is_sac_file)  then
c           -- have to close input file
            close( 3 )
c           -- read sac-file, on auto-detect this is second read attempt
            call sacin( infile, iy, ierr )
         else if (is_seed_file) then
c           -- have to close input file
            close( 3 )
c           -- read miniSEED-file, on auto-detect this is second read attempt
            call mseedin( infile, iy, ierr )
         else if (is_css_file) then
            call cssin( 3, iy, ierr )
            if  (ierr .ne. 0)  goto 999
         else
            call gsein( 3, 4, cbuf, iy, ichecksum, ierr )
            if  (ierr .ne. 0)  goto 999
         endif

         iwaveform = iwaveform + 1

c        -- write the trace to output file
         if  (hdr_ofmtnum .gt. 0)  then
            call gseout( 4, cbuf, iy, ichecksum )
         else if  ((hdr_ofmtnum .eq. -1) .or. (hdr_ofmtnum .eq. -2))
     &      then
            call sacout( outfile, iy, iwaveform )
         else if  (hdr_ofmtnum .eq. -3)  then
            call mseedout( iy )
         else if  ((hdr_ofmtnum .eq. -11) .or. (hdr_ofmtnum .eq. -12))
     &      then
            call cssout( 4, outfile, iy, iwaveform)
         else
            write (*,*) '** Illegal format ID', hdr_ofmtnum
            stop
         endif

c        -- exit on SAC input, because only one trace in file
         if  (is_sac_file)  goto 999
         if  (is_seed_file)  goto 999

      goto 10
999   continue
c     -- end of loop over traces

c     -- close input and output files
      if  ((hdr_ofmtnum .gt. 0) .or. (hdr_ofmtnum .lt. -10))  then
         close(4)
      else
         close(4,status='delete')
      endif
      if  (.not. is_sac_file)  close(3)

      if  (hdr_debug .gt. 0)  write(*,*)
     &   '-- number of waveforms processed:', iwaveform
c
      end ! of program codeco
