#! /bin/csh
#
# file conf.csh
#      ========
#
# version 2, 24-Aug-98
#
# Configures source and Makefiles
# K. Stammler, 19-Aug-98

if  ("$1" == "-h" || "$1" == "--help")  then
	echo "Usage: $0 [<compiler>] [<system>]"
	exit
endif

# get parameters
set compiler=$1
set sysname=$2

# determine architecture
if  ("$sysname" == "")  then
	set sysname=`uname`
	echo "found system $sysname"
else
	echo "use system $sysname"
endif

# determine compiler if not specified
if  ("$compiler" == "")  then
	set clist = ( f77 g77 f90 )
	if  ("$sysname" == "SunOS")  then
		foreach c ($clist)
			set res=`which $c`
			if   ("`echo $res | grep 'not found'`" == "")  then
				if  ($#res > 0)  then
					if  ($res[1] != "no")  then
						set compiler=$c
						break
					endif
				endif
			endif
		end
	else
		foreach c ($clist)
			set res=`whereis $c`
			if   ($#res > 1)  then
				set compiler=$c
				# add to path if not yet done
				set compath=$res[2]
				set compath=$compath:h
				set tmp=`echo $path | grep $compath`
				if  ("$tmp" == "")  set path = ( $path $compath )
				break
			endif
		end
	endif
	echo "found compiler $compiler"
else
	echo "use compiler $compiler"
endif

# set compiler flags and conditional code
set fflags=""       # default is no options
set hp_code=""      # enabled
set sun_code="c"    # disabled
set linux_code="c"  # disabled
if  ("$sysname" == "HP-UX")  then
	set hp_code=""
	set sun_code="c"
	set linux_code="c"
	if  ("$compiler" == "f77")  then
		set fflags="+E4 -K"
	else if  ("$compiler" == "f90")  then
		set fflags="+save +DAportable"
	endif
else if  ("$sysname" == "Linux")  then
	set hp_code="c"
	set sun_code="c"
	set linux_code=""
	if  ("$compiler" == "g77")  then
		set fflags=""
	endif
else if  ("$sysname" == "SunOS")  then
	set hp_code="c"
	set sun_code=""
	set linux_code="c"
	if  ("$compiler" == "f77")  then
		set fflags=""
	else if  ("$compiler" == "g77")  then
		set fflags=""
	endif
else
	echo "$0 : not yet configured for $sysname.  Expect core dumps."
endif

# insert compiler name to Makefile(s)
set f_clist = ( Makefile )
foreach f ($f_clist)
	if  (! -e $f.templ)  then
		echo "Cannot find template $f.templ.  Abort."
		exit
	endif
	if  (-e $f)  \rm $f
	sed "s/--COMPILER--/$compiler/g" $f.templ \
		| sed "s/--FFLAGS--/$fflags/g" >$f
end

# machine dependent code
set f_clist = ( compdecomp.f )
foreach f ($f_clist)
	if  (! -e $f.templ)  then
		echo "Cannot find template $f.templ from `pwd`.  Abort."
		exit
	endif
	if  (-e $f)  \rm $f
	sed "s/--SUN--/$sun_code/g" $f.templ \
		| sed "s/--HP--/$hp_code/g" \
		| sed "s/--LINUX--/$linux_code/g" >$f
end

# put absolute path to scripts
set f_clist = ( ../bin/call_codeco.csh ../bin/uico.csh )
# get root path (now we are in 'src' path)
cd ..
set rootpath=`pwd`
cd src
# loop all files
foreach f ($f_clist)
	if  (! -e $f.templ)  then
		echo "Cannot find template $f.templ from `pwd`.  Abort."
		exit
	endif
	if  (-e $f)  \rm $f
	set replpath=`echo $rootpath | sed 's/\//\\\//g'`
	sed "s/--ROOTPATH--/$replpath/g" $f.templ >$f
	chmod a+x $f
end


# create link to write_steim1
cd ../bin
if  (-e write_steim1)  \rm write_steim1
if  (-e ../fixed_bin/write_steim1.$sysname)  then
	ln -s ../fixed_bin/write_steim1.$sysname write_steim1
endif

