      SUBROUTINE UTMEPO( YEAR, UTM ,EPOTIM )
 
      INTEGER*4  YEAR   , UTM(5)
      REAL*8     EPOTIM
 
C.======================================================================
C.    PURPOSE
C     Utmepo                                                       EPO<<
C     Conversion from GMT human time to epoch time.                
C     Conversion between human time and CSS epoch time.              
C     Epoch time is double prececision floating point seconds from   
C     day one 1970. (I.e. epoch time 0.000 is day one 00:00:00 1970. 
C.----------------------------------------------------------------------
C.    KEYWORDS
C.----------------------------------------------------------------------
C.    PACKAGE
C.    VISIBLE
C.    STANDARD_FORTRAN_77
C.    USE_ONLY
C.----------------------------------------------------------------------
C.    INPUT
C
C..   YEAR    - Year ( ...,1970,...,1987,...).
C..   UTM     - Day-of-year,hour,minutes,seconds,deciseconds.
C.              UTM(1) = day-of-year (1,2,3,...,365,(366)).
C.              UTM(2) = hours       (00,...,23).
C.              UTM(3) = minutes     (00,...,59).
C.              UTM(4) = seconds     (00,...,59).
C.              UTM(5) = milliseconds (0,...,999).
C.              If all UTM's are zero, then EPOTIM is zero.
C
C.    INPUT_IN_COMMON
C
C.    OUTPUT
C
C..   EPOTIM  - Epoch time in seconds. Positive or negative values.
C.              ( 0.000 = 1970,001,00,00,00,0 (day 1,1970).
C
C.    OUTPUT_IN_COMMON
C.----------------------------------------------------------------------
C.    PROGRAMMER    Jan Fyen
C.    CREATION_DATE 1987
C.    MADE_AT  NTNF/NORSAR
C     Pb. 51
C     N-2007 Kjeller
C
C.    MODIFICATION  Converted from Science Horizions c-program htoe
C.    CORRECTION
C.======================================================================
 
C.OPT(3) 
      INTEGER*4 DAYS
      REAL*8    SEC  , EPODAT
 
      EPOTIM  = 0.0D0
C                           Error on input if UTM's < 0
      IF( UTM(1).LT.0 .OR.
     +    UTM(2).LT.0 .OR.
     +    UTM(3).LT.0 .OR.
     +    UTM(4).LT.0 .OR.
     +    UTM(5).LT.0      ) RETURN
 
C                           All UTM's = 0 are accepted as EPOTIM = 0
      IF( UTM(1)+UTM(2)+UTM(3)+UTM(4)+UTM(5).EQ.0 ) RETURN
 
C                           Error on input if UTM(1) < 1
      IF( UTM(1).LT.1 ) RETURN
 
      DAYS   = 0
      IF( YEAR .GT.1970 ) THEN
          DO 100 IYEAR = 1970 , YEAR - 1
             DAYS = DAYS+365
             IF( (IYEAR - (IYEAR/4)*4)    .EQ.0 .AND.
     +           (IYEAR - (IYEAR/100)*100).NE.0 .OR.
     +           (IYEAR - (IYEAR/400)*400).EQ.0        ) DAYS = DAYS+1
100       CONTINUE
      END IF
      IF( YEAR .LT.1970 ) THEN
          DO 200 IYEAR = YEAR , 1969
             DAYS = DAYS-365
             IF( (IYEAR - (IYEAR/4)*4)    .EQ.0 .AND.
     +           (IYEAR - (IYEAR/100)*100).NE.0 .OR.
     +           (IYEAR - (IYEAR/400)*400).EQ.0        ) DAYS = DAYS-1
200       CONTINUE
      END IF
      DAYS   = DAYS + UTM(1) - 1
      EPODAT = DAYS
      EPODAT = EPODAT*86400.0D0
      SEC    = UTM(2)*3600.0D0+UTM(3)*60.0D0+UTM(4)+UTM(5)/1000.0D0
      EPOTIM = EPODAT + SEC
 
      RETURN
      END


